//! This module helps us deal with controls and events.

use std::cell::RefCell;
use std::collections::{HashMap, VecDeque};
use std::rc::Rc;

use wasm_bindgen::prelude::*;
use wasm_bindgen::JsCast;

use crate::Result;

/// The different actions that a user can do.
#[derive(Copy, Clone)]
pub enum Event {
    /// A button has been pressed.
    ButtonPressed(Button),

    /// A button has been released.
    ButtonReleased(Button),
}

/// The different actions that a user can do.
#[derive(Copy, Clone)]
pub enum Button {
    /// The user asks to go to the left.
    Left,

    /// The user asks to go to the right.
    Right,

    /// The user asks to go up.
    Up,

    /// The user asks to go down.
    Down,

    /// The main action or validatation button.
    Button0,

    /// The user asks to do the secondary action.
    Button1,

    /// The user asks to do the tertiary action.
    Button2,

    /// The user asks to do the 4th action or cancel.
    Button3,
}

/// This structure holds what button are pressed or released, and stores the events.
pub struct InputManager {
    /// All the connected gamepads.
    gamepads: Vec<Gamepad>,

    /// The events that have occur.
    events: VecDeque<Event>,
}

impl InputManager {
    /// Creates a new input manager.
    pub fn new() -> InputManager {
        InputManager {
            gamepads: vec![],
            events: VecDeque::new(),
        }
    }
}

/// Holds the gamepad information.
#[derive(Clone)]
pub struct Gamepad {
    /// The inner gamepad.
    inner: web_sys::Gamepad,
}
